package com.xpo.ltl.api.client;

import java.io.IOException;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.XMLGregorianCalendar;

import com.google.gson.JsonParseException;
import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonToken;
import com.google.gson.stream.JsonWriter;
import com.xpo.ltl.api.client.util.DateTimeHelper;

public class XMLGregorianCalendarAdapter extends TypeAdapter<XMLGregorianCalendar> {

	@Override
	public void write(JsonWriter writer, XMLGregorianCalendar value) throws IOException {
		if (value == null) {
			writer.nullValue();
			return;
		}
		writer.value(value.toGregorianCalendar().getTimeInMillis());
	}

	@Override
	public XMLGregorianCalendar read(JsonReader reader) throws IOException {
		JsonToken token = reader.peek();
		if (token == JsonToken.NULL) {
			reader.nextNull();
			return null;
		}

		if(token == JsonToken.NUMBER) {
			long timeInMillis = reader.nextLong();
			try {
				return DateTimeHelper.convertTimeInMillisToXMLGC(timeInMillis);
			} catch (DatatypeConfigurationException e) {
				throw new JsonParseException(e);
			}
		} else {
			String dateTime = reader.nextString();
			try {
				return DateTimeHelper.convertStringToXMLGC(dateTime);
			} catch (Exception e) {
				throw new JsonParseException(e);
			}
		}
	}
}
