package com.xpo.ltl.api.rating.client.v1;

import java.text.MessageFormat;
import java.util.Arrays;
import javax.xml.datatype.XMLGregorianCalendar;

import com.xpo.ltl.api.client.Resource;
import com.xpo.ltl.api.client.ServiceClient;
import com.xpo.ltl.api.client.common.Response;
import com.xpo.ltl.api.client.common.ClientCredentials;
import com.xpo.ltl.api.client.common.Attachment;

import com.xpo.ltl.api.client.exception.ServiceException;
import com.xpo.ltl.api.rating.v1.GetXpoLtlRateQuoteRqst;
import com.xpo.ltl.api.rating.v1.GetXpoLtlRateQuoteResp;
import com.xpo.ltl.api.rating.v1.GetXpoLtlSpotQuoteRqst;
import com.xpo.ltl.api.rating.v1.GetXpoLtlSpotQuoteResp;
import com.xpo.ltl.api.rating.v1.GetXpoLtlRateQuoteDetailsResp;
import com.xpo.ltl.api.rating.v1.ListXpoLtlRateQuotesByAcctResp;
import com.xpo.ltl.api.rating.v1.ListRateQuoteAccessorialsResp;
import com.xpo.ltl.api.rating.v1.CreateCurrencyExchangeRateRqst;
import com.xpo.ltl.api.rating.v1.GetRatingMessageResp;
import com.xpo.ltl.api.rating.v1.SaveRatingMessageRqst;
import com.xpo.ltl.api.client.common.ServiceInfo;
import com.xpo.ltl.api.rating.v1.User;

public class RatingClient extends ServiceClient
{
	// Private constructor. Use newInstance(...) to create an instance
	private RatingClient() {
	}

	public static RatingClient newInstance(String host, String basePath, ClientCredentials credentials) throws ServiceException {
		return newInstance(host, basePath, credentials, new RatingClient());
	}
	
	public static RatingClient newInstance(String host, String basePath, String accessToken) throws ServiceException {
		return newInstance(host, basePath, accessToken, new RatingClient());
	}
	
	public static String version() {
		return "1.0-RC_b343";
	}
	
	/**
	* Health check URL. Responds with success message if the service is running.
	*/
	public Response<ServiceInfo> healthCheck() throws ServiceException {
		String resourcePath = "/health-check";
		Resource<ServiceInfo> resource = new Resource<ServiceInfo>(host, basePath, resourcePath, getToken(), ServiceInfo.class);
		return resource.get();
	}
	/**
	* List of resources.
	*/
	public Response<String> options() throws ServiceException {
		String resourcePath = "/options";
		Resource<String> resource = new Resource<String>(host, basePath, resourcePath, getToken(), String.class);
		return resource.get();
	}
	/**
	* Details of user invoking the API.
	*/
	public Response<User> loggedInUser() throws ServiceException {
		String resourcePath = "/appusers/logged-in-user";
		Resource<User> resource = new Resource<User>(host, basePath, resourcePath, getToken(), User.class);
		return resource.get();
	}
	/**
	* This service operation will create Rate Quote based on input shipment information and the Customer Account Pricing Agreement. Rate Quote details will be sent back in response.
	* <br/>Pre Condition
	* <br/>Valid shipment information like pickup date, shipper information, consignee information, bill-to party information, list of accessorial codes, if any, and commodities being shipped is supplied.
	* <br/>Post Condition
	* <br/>A Rate Quote is created and rate quote details including the rate quote confirmation number and applicable charge amounts are returned back to the consumer.
	* <br/>
	*/
	public Response<GetXpoLtlRateQuoteResp> getXpoLtlRateQuote(GetXpoLtlRateQuoteRqst getXpoLtlRateQuoteRqst) throws ServiceException {
		String resourcePath = "/ratequotes";
		Resource<GetXpoLtlRateQuoteResp> resource = new Resource<GetXpoLtlRateQuoteResp>(host, basePath, resourcePath, getToken(), GetXpoLtlRateQuoteResp.class);
		return resource.post(getXpoLtlRateQuoteRqst);
	}
	/**
	* This service operation will calculate and create spot quote based on input shipment information. Spot quote number will be sent back in response.
	* <br/>Pre Condition
	* <br/>Valid shipment information like pickup date, shipper information, consignee information and commodities being shipped is supplied and a spot rate is available for the requested lane.
	* <br/>Post Condition
	* <br/>A spot quote is created and spot quote information including the spot quote number and amounts are returned back to the consumer.
	* <br/>Pre Condition
	* <br/>Valid shipment information like pickup date, shipper information, consignee information and commodities being shipped is supplied and a spot rate is not available for the requested lane.
	* <br/>Post Condition
	* <br/>Spot quote is not created and error message is returned.
	* <br/>
	*/
	public Response<GetXpoLtlSpotQuoteResp> getXPOLTLSpotQuote(GetXpoLtlSpotQuoteRqst getXpoLtlSpotQuoteRqst) throws ServiceException {
		String resourcePath = "/spotquotes";
		Resource<GetXpoLtlSpotQuoteResp> resource = new Resource<GetXpoLtlSpotQuoteResp>(host, basePath, resourcePath, getToken(), GetXpoLtlSpotQuoteResp.class);
		return resource.post(getXpoLtlSpotQuoteRqst);
	}
	/**
	* This service will retrieve the details of an existing rate quote based on its confirmation number. Rate Quote details will be sent back in response.
	* <br/>Pre Condition
	* <br/>1. A valid rate quote confirmation number is provided.
	* <br/>2. Invalid rate quote confirmation number is provided.
	* <br/>Post Condition
	* <br/>1. Rate quote details for input rate quote confirmation number is returned.
	* <br/>2. A 'Not found' error message is returned.
	* <br/>
	*/
	public Response<GetXpoLtlRateQuoteDetailsResp> getXpoLtlRateQuoteDetails(String confirmationNbr) throws ServiceException {
		String resourcePath = "/ratequotes/" + confirmationNbr.toString().replaceAll(" ", "%20") + "";
		Resource<GetXpoLtlRateQuoteDetailsResp> resource = new Resource<GetXpoLtlRateQuoteDetailsResp>(host, basePath, resourcePath, getToken(), GetXpoLtlRateQuoteDetailsResp.class);
		return resource.get();
	}
	/**
	* This API will retrieve a list of all the Rate Quotes for input XPO LTL Account and given date range.
	* <br/>Pre Condition
	* <br/>	A valid XPO LTL Customer Account number is provided and rate quotes exist for the input date range.
	* <br/>Post Condition
	* <br/>	List of all the rate quotes for the input customer account is returned.
	* <br/>Pre Condition
	* <br/>	A valid XPO LTL Customer Account Number is provided but no rate quotes found for the input date range
	* <br/>Post Condition	
	* <br/>	Not found (status code = 404) message is returned to the consumer.
	* <br/>Pre Condition
	* <br/>	Invalid XPO LTL Customer Account is provided
	* <br/>Post Condition	
	* <br/>	Invalid Account Number (Status code = 400) message is returned to the customer.
	* <br/>
	*/
	public Response<ListXpoLtlRateQuotesByAcctResp> ListXpoLtlRateQuotesByAcct(String acctInstId, XMLGregorianCalendar rangeStartDate, XMLGregorianCalendar rangeEndDate, String levelOfDetail, Integer numberOfRows, Integer startAt, String[] fields, String[] sortByFieldName, String[] sortOrder) throws ServiceException {
		String resourcePath = "/ratequotes";
		Resource<ListXpoLtlRateQuotesByAcctResp> resource = new Resource<ListXpoLtlRateQuotesByAcctResp>(host, basePath, resourcePath, getToken(), ListXpoLtlRateQuotesByAcctResp.class);
		resource.addParameter("acctInstId", acctInstId);
		resource.addParameter("rangeStartDate", rangeStartDate);
		resource.addParameter("rangeEndDate", rangeEndDate);
		resource.addParameter("levelOfDetail", levelOfDetail);
		resource.addParameter("numberOfRows", numberOfRows);
		resource.addParameter("startAt", startAt);
		resource.addParameter("fields[]", fields);
		resource.addParameter("sortByFieldName[]", sortByFieldName);
		resource.addParameter("sortOrder[]", sortOrder);
		return resource.get();
	}
	/**
	* This API will retrieve a list of accessorial codes that should be used for Rate Quote.
	* <br/>Pre Condition
	* <br/>	typeCode = ratequote is specified in the request
	* <br/>Post Condition
	* <br/>	List of all the accessorials to be used for rate quote are returned.
	* <br/>
	* <br/>Pre Condition
	* <br/>	Invalid typeCode value or typeCode not specified.
	* <br/>Post Condition
	* <br/>	Error code 400: Invalid input is returned.
	* <br/>
	* <br/>
	*/
	public Response<ListRateQuoteAccessorialsResp> listRateQuoteAccessorials(String levelOfDetail, Integer numberOfRows, Integer startAt, String[] fields, String[] sortByFieldName, String[] sortOrder, String origCountry, String destCountry) throws ServiceException {
		String resourcePath = "/accessorials/ratequote";
		Resource<ListRateQuoteAccessorialsResp> resource = new Resource<ListRateQuoteAccessorialsResp>(host, basePath, resourcePath, getToken(), ListRateQuoteAccessorialsResp.class);
		resource.addParameter("levelOfDetail", levelOfDetail);
		resource.addParameter("numberOfRows", numberOfRows);
		resource.addParameter("startAt", startAt);
		resource.addParameter("fields[]", fields);
		resource.addParameter("sortByFieldName[]", sortByFieldName);
		resource.addParameter("sortOrder[]", sortOrder);
		resource.addParameter("origCountry", origCountry);
		resource.addParameter("destCountry", destCountry);
		return resource.get();
	}
	/**
	* Creates a currency exchange for a specified Currency Code and Target Currency Code combination and Effective Date.
	* <br/>PreCondition
	* <br/>valid values is entered: Currency Code, Target Currency Code, Exchange Rate and Effective Date.
	* <br/>Post Condition:
	* <br/>Create the currency exchange for the above specified details.
	*/
	public Response<Void> createCurrencyExchangeRate(CreateCurrencyExchangeRateRqst createCurrencyExchangeRateRqst) throws ServiceException {
		String resourcePath = "/exchangerates";
		Resource<Void> resource = new Resource<Void>(host, basePath, resourcePath, getToken(), Void.class);
		return resource.post(createCurrencyExchangeRateRqst);
	}
	/**
	* This service will retrieve the rating message based on the rating type (PreviewCorrections, ApplyCorrections, InvoicePass, other passes.....) and type of message to be retrieved (Request or Response) for a given Shipment Id or ProNumber and Pickup Date.
	* <br/>Pre-conditions: 
	* <br/>1. Shipment Id or ProNumber and PickupDate combination is provided and Rating type and Message type is provided and the message exists for the given combination.
	* <br/>Post-conditions: 
	* <br/>1. Appropriate message is retrieved for the given shipment, rating type and message type.
	* <br/>Pre-conditions: 
	* <br/>2. No data exists for given Shipment for the given Rating type and Message type.
	* <br/>Post-conditions: 
	* <br/>2. Not found message is returned.
	* <br/>
	*/
	public Response<GetRatingMessageResp> getRatingMessage(String ratingType, Integer shipmentInstID, String proNbr, XMLGregorianCalendar pkupDateTime, String messageType) throws ServiceException {
		String resourcePath = "/ratingmessages";
		Resource<GetRatingMessageResp> resource = new Resource<GetRatingMessageResp>(host, basePath, resourcePath, getToken(), GetRatingMessageResp.class);
		resource.addParameter("ratingType", ratingType);
		resource.addParameter("shipmentInstID", shipmentInstID);
		resource.addParameter("proNbr", proNbr);
		resource.addParameter("pkupDateTime", pkupDateTime);
		resource.addParameter("messageType", messageType);
		return resource.get();
	}
	/**
	* This service will save the rating message based on the rating type (PreviewCorrections, ApplyCorrections, InvoicePass, other passes.....) and type of message to be saved (Request or Response) for a given Shipment Id or ProNumber and Pickup Date.
	* <br/>Pre-conditions: 
	* <br/> Shipment Id or ProNumber and PickupDate combination is provided and Rating type and Message type is provided.
	* <br/>Post-conditions: 
	* <br/> Data is inserted into the table and Success status code - 201 is returned.
	* <br/>
	*/
	public Response<Void> saveRatingMessage(SaveRatingMessageRqst saveRatingMessageRqst) throws ServiceException {
		String resourcePath = "/ratingmessages";
		Resource<Void> resource = new Resource<Void>(host, basePath, resourcePath, getToken(), Void.class);
		return resource.post(saveRatingMessageRqst);
	}

}
